@@ -102,7 +102,7 @@ typedef struct kdnode
 										// ..   hiIx is index corresponding to
 										// ..   cutVal
 
-	// the following fields are onlyl valid if isBucket is false
+	// the following fields are only valid if isBucket is false
 
 	sgnpos		cutVal;					// value (along appropriate axis) which
 										// .. separates lower and upper children
@@ -662,7 +662,7 @@ static kdnode* build_kd_tree
 		p->loIx = lo;
 		p->hiIx = hi;
 		}
-	else							// the range is two big for one node, split
+	else							// the range is too big for one node, split
 		{							// .. it into two subtrees
 		p->isBucket = false;
 		m = partition_segments (lo, hi, axis, kdi);
