@@ -2095,7 +2095,7 @@ void dump_score_set
 	u8*			r, *c;
 
 	if (rowChars == NULL) rowChars = ss->rowChars;
-	if (colChars == NULL) rowChars = ss->colChars;
+	if (colChars == NULL) colChars = ss->colChars;
 
 	if ((dna_utilities_scoreType == 'F') || (dna_utilities_scoreType == 'D'))
 		width = 13;
@@ -2860,6 +2860,11 @@ char* bits_to_nuc_string
 //
 // WARNING:  These functions are only valid for DNA sequences.
 //
+//
+// Note: This is different than the function sequence_entropy() in sequences.c.
+//       This function computes the entropy of an ungapped alignment, while
+//       that function computes the entropy of a single sequence segment.
+//
 //----------
 //
 // Arguments:
