@@ -173,6 +173,8 @@ void print_genpaf_job_header
 			case genpafTargetQuals:      fprintf (f, "quals1");             break;
 			case genpafQueryNucs:        fprintf (f, "nucs2");              break;
 			case genpafQueryQuals:       fprintf (f, "quals2");             break;
+			case genpafEntropy1:         fprintf (f, "entropy1");           break;
+			case genpafEntropy2:         fprintf (f, "entropy2");           break;
 			case genpafHspId:            fprintf (f, "hspid");              break;
 			case genpafPositionHash:     fprintf (f, "phash");              break;
 			case genpafAlignmentHash:    fprintf (f, "ahash");              break;
@@ -591,6 +593,7 @@ void print_genpaf_align
 	sgnpos			diag, diagSE, diagNW;
 	unspos			numGaps;
 	u32				h;
+	float			entropy;
 
 	snoopGenpaf_5;
 
@@ -1100,6 +1103,7 @@ void print_genpaf_align
 				                   /* withInfo       */ false,
 				                   /* markMismatches */ true,
 				                   /* letterAfter    */ true,
+				                   /* withSpaces     */ true,
 				                   /* hideSingles    */ true,
 				                   /* lowerCase      */ (*k == genpafCigarXLower),
 				                   /* withNewLine    */ false);
@@ -1111,6 +1115,7 @@ void print_genpaf_align
 				                   /* withInfo       */ false,
 				                   /* markMismatches */ true,
 				                   /* letterAfter    */ true,
+				                   /* withSpaces     */ true,
 				                   /* hideSingles    */ false,
 				                   /* lowerCase      */ (*k == genpafCigarXLower),
 				                   /* withNewLine    */ false);
@@ -1260,6 +1265,16 @@ void print_genpaf_align
 						fprintf (f, "%c", seq2->vq[endOffset2-ix]);
 					}
 				break;
+			case genpafEntropy1:
+				entropy = (float) sequence_entropy (seq1, beg1, end1-beg1);
+				if (entropy >= 0.0) fprintf (f, "%0.3f", entropy);
+				else                fprintf (f, "NA");
+				break;
+			case genpafEntropy2:
+				entropy = (float) sequence_entropy (seq2, beg2, end2-beg2);
+				if (entropy >= 0.0) fprintf (f, "%0.3f", entropy);
+				else                fprintf (f, "NA");
+				break;
 			case genpafHspId:
 				fprintf (f, u64Fmt, hspId);
 				break;
@@ -1290,6 +1305,7 @@ void print_genpaf_align
 				                   /* withInfo       */ false,
 				                   /* markMismatches */ true,
 				                   /* letterAfter    */ true,
+				                   /* withSpaces     */ true,
 				                   /* hideSingles    */ false,
 				                   /* lowerCase      */ false,
 				                   /* withNewLine    */ false);
@@ -1301,6 +1317,7 @@ void print_genpaf_align
 				                   /* withInfo       */ false,
 				                   /* markMismatches */ false,
 				                   /* letterAfter    */ true,
+				                   /* withSpaces     */ true,
 				                   /* hideSingles    */ false,
 				                   /* lowerCase      */ false,
 				                   /* withNewLine    */ false);
@@ -1387,6 +1404,7 @@ void print_genpaf_match
 	char*			textDiffInfo;
 	sgnpos			diag, diagSE, diagNW;
 	u32				h;
+	float			entropy;
 
 	snoopGenpaf_6;
 
@@ -1667,6 +1685,7 @@ void print_genpaf_match
 				                   /* withInfo       */ false,
 				                   /* markMismatches */ true,
 				                   /* letterAfter    */ true,
+				                   /* withSpaces     */ true,
 				                   /* hideSingles    */ true,
 				                   /* lowerCase      */ (*k == genpafCigarXLower),
 				                   /* withNewLine    */ false);
@@ -1678,6 +1697,7 @@ void print_genpaf_match
 				                   /* withInfo       */ false,
 				                   /* markMismatches */ true,
 				                   /* letterAfter    */ true,
+				                   /* withSpaces     */ true,
 				                   /* hideSingles    */ false,
 				                   /* lowerCase      */ (*k == genpafCigarXLower),
 				                   /* withNewLine    */ false);
@@ -1848,6 +1868,16 @@ void print_genpaf_match
 						fprintf (f, "%c", seq2->vq[endOffset2-ix]);
 					}
 				break;
+			case genpafEntropy1:
+				entropy = (float) sequence_entropy (seq1, pos1, length);
+				if (entropy >= 0.0) fprintf (f, "%0.3f", entropy);
+				else                fprintf (f, "NA");
+				break;
+			case genpafEntropy2:
+				entropy = (float) sequence_entropy (seq2, pos2, length);
+				if (entropy >= 0.0) fprintf (f, "%0.3f", entropy);
+				else                fprintf (f, "NA");
+				break;
 			case genpafHspId:
 				fprintf (f, u64Fmt, hspId);
 				break;
@@ -1874,6 +1904,7 @@ void print_genpaf_match
 				                   /* withInfo       */ false,
 				                   /* markMismatches */ true,
 				                   /* letterAfter    */ true,
+				                   /* withSpaces     */ true,
 				                   /* hideSingles    */ false,
 				                   /* lowerCase      */ false,
 				                   /* withNewLine    */ false);
@@ -1885,6 +1916,7 @@ void print_genpaf_match
 				                   /* withInfo       */ false,
 				                   /* markMismatches */ false,
 				                   /* letterAfter    */ true,
+				                   /* withSpaces     */ true,
 				                   /* hideSingles    */ false,
 				                   /* lowerCase      */ false,
 				                   /* withNewLine    */ false);
